package com.ningdatech.pmapi.projectlib.manage;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectRenewalApprovalStatusEnum;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectRenewalFundDeclarationDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectRenewalFundDeclaration;
import com.ningdatech.pmapi.projectlib.model.req.ProjectRenewalListReq;
import com.ningdatech.pmapi.projectlib.model.vo.ProjectRenewalFundDeclarationVO;
import com.ningdatech.pmapi.projectlib.service.IProjectRenewalFundDeclarationService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.wflow.exception.BusinessException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * ProjectRenewalFundManage
 * </p>
 *
 * @author ZPF
 * @since 14:19 2023/2/15
 */
@Component
@RequiredArgsConstructor
public class ProjectRenewalFundManage {

    private final IProjectRenewalFundDeclarationService projectRenewalFundDeclarationService;

    private final IProjectService projectService;

    /**
     * 项目续建资金库列表 分页
     * @param req
     * @return
     */
    public PageVo<ProjectRenewalFundDeclarationVO> list(ProjectRenewalListReq req) {
        Page<ProjectRenewalFundDeclaration> page = req.page();
        projectRenewalFundDeclarationService.pageSql(page, req);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        List<ProjectRenewalFundDeclarationVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectRenewalFundDeclarationVO item = new ProjectRenewalFundDeclarationVO();
            BeanUtils.copyProperties(w,item);
            return item;
        });
        return PageVo.of(records, total);
    }

    /**
     * @param id
     * @author ZPF
     * @since 2023/02/15 11:15
     */
    public ProjectRenewalFundDeclarationVO detail(Long id) {
        // 查询项目申报信息
        ProjectRenewalFundDeclaration renewal = projectRenewalFundDeclarationService.getById(id);
        if(Objects.isNull(renewal)){
            return null;
        }
        ProjectRenewalFundDeclarationVO vo = new ProjectRenewalFundDeclarationVO();
        BeanUtils.copyProperties(renewal, vo);
        // 查询项目信息
        if(Objects.nonNull(vo)){
            Project project = projectService.getById(renewal.getProjectId());
            if(Objects.isNull(project)){
                return vo;
            }
            vo.setApprovalAmount(project.getApprovalAmount());
            vo.setProjectName(project.getProjectName());
            vo.setProjectType(project.getProjectType());
            vo.setStage(project.getStage());
            vo.setStatus(project.getStatus());
        }
        return vo;
    }

    /**
     * 申报
     * @param dto
     * @return
     */
    public Long declared(ProjectRenewalFundDeclarationDTO dto) {
        ProjectRenewalFundDeclaration declaration = new ProjectRenewalFundDeclaration();
        BeanUtils.copyProperties(dto,declaration);
        declaration.setId(null);
        declaration.setCreateOn(LocalDateTime.now());
        declaration.setUpdateOn(LocalDateTime.now());
        declaration.setApprovalStatus(ProjectRenewalApprovalStatusEnum.PENDING.name());
        if(!projectRenewalFundDeclarationService.save(declaration)){
            throw new BusinessException("申报失败");
        }
        return declaration.getId();
    }
}
