package com.ningdatech.pmapi.user.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.exception.BizException;
import com.ningdatech.pmapi.organization.model.entity.DingEmployeeInfo;
import com.ningdatech.pmapi.organization.model.entity.DingOrganization;
import com.ningdatech.pmapi.organization.service.IDingEmployeeInfoService;
import com.ningdatech.pmapi.organization.service.IDingOrganizationService;
import com.ningdatech.pmapi.user.entity.UserInfo;
import com.ningdatech.pmapi.user.mapper.NdUserInfoMapper;
import com.ningdatech.pmapi.user.service.IUserInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.wflow.workflow.bean.vo.UserInfoVO;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author Lierbao
 * @since 2023-02-01
 */
@Service
@RequiredArgsConstructor
public class UserInfoServiceImpl extends ServiceImpl<NdUserInfoMapper, UserInfo> implements IUserInfoService {

    private final NdUserInfoMapper userInfoMapper;
    private final IDingEmployeeInfoService dingEmployeeInfoService;

    private final IDingOrganizationService dingOrganizationService;
    @Override
    public Map<Long, UserInfoVO> getUserMapByIds(Set<String> staterUsers) {
        List<UserInfo> userInfos = userInfoMapper.selectBatchIds(staterUsers);
        List<UserInfoVO> resVos = userInfos.stream().map(u -> {
            UserInfoVO userInfoVO = new UserInfoVO();
            userInfoVO.setUserId(u.getId());
            userInfoVO.setRealName(u.getRealName());
            userInfoVO.setAccountId(u.getAccountId());
            Long accountId = u.getAccountId();
            if (Objects.isNull(accountId)){
                throw new BizException("该用户没有录入浙政钉用户信息!");
            }
            // 根据浙政钉用户ID获取部门code
            DingEmployeeInfo employeeInfo = dingEmployeeInfoService.getOne(Wrappers.lambdaQuery(DingEmployeeInfo.class)
                    .eq(DingEmployeeInfo::getId, accountId));
            String organizationCode = employeeInfo.getOrganizationCode();
            // 根据部门code获取部门名称
            DingOrganization dingOrganization = dingOrganizationService.getOne(Wrappers.lambdaQuery(DingOrganization.class)
                    .eq(DingOrganization::getOrganizationCode, organizationCode));
            String organizationName = dingOrganization.getOrganizationName();
            userInfoVO.setOrganizationCode(organizationCode);
            userInfoVO.setOrganizationName(organizationName);
            return userInfoVO;
        }).collect(Collectors.toList());
        return resVos.stream().collect(Collectors.toMap(UserInfoVO::getUserId, v -> v));
    }
}
